@@ -1,3 +1,8 @@
+{-# LANGUAGE DeriveAnyClass #-}
+{-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE DeriveGeneric #-}
+{-# LANGUAGE Safe #-}
+
 -- |
 -- Module      :  Text.Megaparsec.Pos
 -- Copyright   :  © 2015–present Megaparsec contributors
@@ -12,23 +17,20 @@
 --
 -- You probably do not want to import this module directly because
 -- "Text.Megaparsec" re-exports it anyway.
-
-{-# LANGUAGE DeriveDataTypeable         #-}
-{-# LANGUAGE DeriveGeneric              #-}
-{-# LANGUAGE GeneralizedNewtypeDeriving #-}
-
 module Text.Megaparsec.Pos
   ( -- * Abstract position
-    Pos
-  , mkPos
-  , unPos
-  , pos1
-  , defaultTabWidth
-  , InvalidPosException (..)
+    Pos,
+    mkPos,
+    unPos,
+    pos1,
+    defaultTabWidth,
+    InvalidPosException (..),
+
     -- * Source position
-  , SourcePos (..)
-  , initialPos
-  , sourcePosPretty )
+    SourcePos (..),
+    initialPos,
+    sourcePosPretty,
+  )
 where
 
 import Control.DeepSeq
@@ -46,15 +48,13 @@ import GHC.Generics
 -- together.
 --
 -- @since 5.0.0
-
 newtype Pos = Pos Int
-  deriving (Show, Eq, Ord, Data, Typeable, NFData)
+  deriving (Show, Eq, Ord, Data, Generic, Typeable, NFData)
 
 -- | Construction of 'Pos' from 'Int'. The function throws
 -- 'InvalidPosException' when given a non-positive argument.
 --
 -- @since 6.0.0
-
 mkPos :: Int -> Pos
 mkPos a =
   if a <= 0
@@ -65,7 +65,6 @@ mkPos a =
 -- | Extract 'Int' from 'Pos'.
 --
 -- @since 6.0.0
-
 unPos :: Pos -> Int
 unPos (Pos w) = w
 {-# INLINE unPos #-}
@@ -73,7 +72,6 @@ unPos (Pos w) = w
 -- | Position with value 1.
 --
 -- @since 6.0.0
-
 pos1 :: Pos
 pos1 = mkPos 1
 
@@ -86,7 +84,6 @@ pos1 = mkPos 1
 -- > defaultTabWidth = mkPos 8
 --
 -- @since 5.0.0
-
 defaultTabWidth :: Pos
 defaultTabWidth = mkPos 8
 
@@ -98,20 +95,21 @@ instance Read Pos where
   readsPrec d =
     readParen (d > 10) $ \r1 -> do
       ("Pos", r2) <- lex r1
-      (x,     r3) <- readsPrec 11 r2
+      (x, r3) <- readsPrec 11 r2
       return (mkPos x, r3)
 
 -- | The exception is thrown by 'mkPos' when its argument is not a positive
 -- number.
 --
 -- @since 5.0.0
-
-newtype InvalidPosException = InvalidPosException Int
-  -- ^ Contains the actual value that was passed to 'mkPos'
+newtype InvalidPosException
+  = -- | Contains the actual value that was passed to 'mkPos'
+    InvalidPosException Int
   deriving (Eq, Show, Data, Typeable, Generic)
 
 instance Exception InvalidPosException
-instance NFData    InvalidPosException
+
+instance NFData InvalidPosException
 
 ----------------------------------------------------------------------------
 -- Source position
@@ -120,28 +118,29 @@ instance NFData    InvalidPosException
 -- name of the source file, a line number, and a column number. Source line
 -- and column positions change intensively during parsing, so we need to
 -- make them strict to avoid memory leaks.
-
 data SourcePos = SourcePos
-  { sourceName   :: FilePath -- ^ Name of source file
-  , sourceLine   :: !Pos     -- ^ Line number
-  , sourceColumn :: !Pos     -- ^ Column number
-  } deriving (Show, Read, Eq, Ord, Data, Typeable, Generic)
+  { -- | Name of source file
+    sourceName :: FilePath,
+    -- | Line number
+    sourceLine :: !Pos,
+    -- | Column number
+    sourceColumn :: !Pos
+  }
+  deriving (Show, Read, Eq, Ord, Data, Typeable, Generic)
 
 instance NFData SourcePos
 
 -- | Construct initial position (line 1, column 1) given name of source
 -- file.
-
 initialPos :: FilePath -> SourcePos
 initialPos n = SourcePos n pos1 pos1
 
 -- | Pretty-print a 'SourcePos'.
 --
 -- @since 5.0.0
-
 sourcePosPretty :: SourcePos -> String
 sourcePosPretty (SourcePos n l c)
-  | null n    = showLC
+  | null n = showLC
   | otherwise = n <> ":" <> showLC
   where
     showLC = show (unPos l) <> ":" <> show (unPos c)
